/*
 * Copyright (c) 1995, 1996
 *	Sandro Sigala, Brescia, Italy.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Sandro Sigala.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: hash.h,v 1.8 1996/06/28 10:28:14 sandro Exp $
 */

#ifndef _HASH_H_
#define _HASH_H_

#define HASH_TABLE_DEFAULT_SIZE		127

typedef unsigned long (*hashing_function_t)(const void *data, unsigned long size, unsigned long table_size);

struct	hash_table_bucket {
	unsigned long ksize;
	unsigned long dsize;
	void *key;
	void *data;
	struct hash_table_bucket *prev;
	struct hash_table_bucket *next;
};

struct	hash_table {
	unsigned long table_size;
	struct hash_table_bucket **table;
	hashing_function_t hashing_function;
};

extern struct hash_table *hash_table_build(unsigned long size);
extern struct hash_table *hash_table_build_default(void);
extern void hash_table_set_hashing_function(struct hash_table *table, hashing_function_t function);
extern int hash_table_free(struct hash_table *table);
extern int hash_table_store_key(struct hash_table *table, const void *key, unsigned long size);
extern int hash_table_exists(struct hash_table *table, const void *key, unsigned long size);
extern int hash_table_delete(struct hash_table *table, const void *key, unsigned long size);
extern int hash_table_store_dataptr(struct hash_table *table, const void *key, unsigned long ksize, const void *data, unsigned long dsize);
extern int hash_table_store_data(struct hash_table *table, const void *key, unsigned long ksize, const void *data, unsigned long dsize);
extern int hash_table_store(struct hash_table *table, const void *key, unsigned long ksize, const void *data, unsigned long dsize);
extern void *hash_table_fetch(struct hash_table *table, const void *key, unsigned long size);

#endif /* !_HASH_H_ */
