/*
 * Copyright (c) 1995, 1996
 *	Sandro Sigala, Brescia, Italy.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Sandro Sigala.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: cinfodc.c,v 1.10 1996/06/28 15:13:15 sandro Exp $
 */

static char *rcsid = "$Id: cinfodc.c,v 1.10 1996/06/28 15:13:15 sandro Exp $";

#include <stdio.h>
#include <getopt.h>
#include <err.h>

#include "version.h"
#include "cinfodb.h"


static FILE *output_file;

static void
pspaces(int n)
{
	int i;
	for (i = 0; i < n; i++)
		fprintf(output_file, " ");
}

static void
output_variable(struct variable_s *p, int level)
{
	for (; p != NULL; p = p->next) {
		pspaces(level);
		fprintf(output_file, "%s = \"%s\"\n", p->name, p->value);
	}
}

static void
output_object(struct object_s *p, int level)
{
	for (; p != NULL; p = p->next) {
		pspaces(level);	fprintf(output_file, "(\"%s\"\n", p->name);
			if (p->variable_list != NULL)
				output_variable(p->variable_list, level + 4);
			if (p->object_list != NULL)
				output_object(p->object_list, level + 4);
		pspaces(level); fprintf(output_file, ")\n");
	}
}

/*
 * Output the program syntax then exit.
 */
static void
usage(void)
{
	fprintf(stderr, "usage: cinfodc [-V] [-o file] [file...]\n");
	exit(1);
}

int
main(int argc, char **argv)
{
	int i, c;
	struct object_s *op;

	output_file = stdout;

	while ((c = getopt(argc, argv, "o:V")) != EOF)
		switch (c) {
		case 'o':
			if (output_file != stdout)
				fclose(output_file);
			if ((output_file = fopen(optarg, "w")) == NULL)
				err(1, "%s", optarg);
			break;
		case 'V':
			fprintf(stderr, "%s - %s\n", version_string, rcsid);
			exit(0);
		case '?':
		default:
			usage();
			/* NOTREACHED */
		}
	argc -= optind;
	argv += optind;

	if (argc < 1)
		usage();

	for (i = 0; i < argc; i++) {
		op = read_cinfodb(argv[i]);
		output_object(op, 0);
		free_object_list(op);
	}

	if (output_file != stdout)
		fclose(output_file);

	return 0;
}
