/*
 * Copyright (c) 1995, 1996
 *	Sandro Sigala, Brescia, Italy.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Sandro Sigala.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: cinfodb.h,v 1.10 1996/06/28 10:28:28 sandro Exp $
 */

#ifndef _CINFODB_H_
#define _CINFODB_H_

#define CINFODB_VERSION_HIGH	1
#define CINFODB_VERSION_LOW	0

#define CINFO_HEADER \
	0177, 'C', 'I', 'N', 'F', 'O', \
	CINFODB_VERSION_HIGH, CINFODB_VERSION_LOW, \
	0, 0, 0, 0, 0, 0, 0, 0

#define CINFO_HEADER_SIZE	16

struct	object_s {
	char *name;
	struct object_s *next;
	struct object_s *object_list;
	struct variable_s *variable_list;
};

struct	variable_s {
	char *name;
	char *value;
	struct variable_s *next;
};

/* tree.c */
extern struct object_s *new_object(char *);
extern struct object_s *last_object(struct object_s *);
extern struct object_s *link_object(struct object_s *, struct object_s *);
extern struct variable_s *new_variable(char *, char *);
extern struct variable_s *last_variable(struct variable_s *);
extern struct variable_s *link_variable(struct variable_s *, struct variable_s *);
extern int count_objects(struct object_s *);
extern int count_variables(struct variable_s *);
extern void free_object(struct object_s *);
extern void free_variable(struct variable_s *);
extern void free_variable_list(struct variable_s *);
extern void free_object_list(struct object_s *);

/* fileio.c */
extern void write_cinfodb(char *, struct object_s *);
extern struct object_s *read_cinfodb(char *);

#endif /* !_CINFODB_H_ */
