/*
 * Copyright (c) 1995, 1996
 *	Sandro Sigala, Brescia, Italy.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Sandro Sigala.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: lexer.l,v 1.5 1996/06/28 14:14:51 sandro Exp $
 */

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <err.h>

#include "parser.h"

extern int parser_eof;
extern int parser_run;

int lineno = 1;

static int yywrap(void);
static void comment(void);
static void directive(void);
%}

%%

"\n"				{ lineno++; }
[ \t\v\f]			/* eat whitespaces */

"/*"				{ comment(); }

"#"				{ directive(); }

[a-zA-Z_\.][a-zA-Z_\.0-9]*	{ return IDENTIFIER; }

'(\\.|[^\\'])+'			{ return CHARACTER; }
\"(\\.|\\\n|[^\\"])*\"		{ return STRING; }

"%%"				{ if (!parser_run)
					return SECTIONSEP;
				  else {
					parser_eof = 1;
					return 0;
				  }
				}

"%"[a-zA-Z_]+			{ directive(); }

.				{ return yytext[0]; }

%%

static int
yywrap(void)
{
	return 1;
}

static void
comment(void)
{
	int c;

	while ((c = input()) != EOF)
		if (c == '\n')
			lineno++;
		else if (c == '*')
			if ((c = input()) == '/')
				return;
			else if (c == '\n')
				lineno++;
}

static void
directive(void)
{
	int c;

	while ((c = input()) != EOF && c != '\n')
		if (c == '\\' && input() == '\n')
				lineno++;
	lineno++;
}
