/*
 * Copyright (c) 1995, 1996
 *	Sandro Sigala, Brescia, Italy.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Sandro Sigala.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tree.c,v 1.3 1996/06/28 11:10:59 sandro Exp $
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <err.h>

#include "tree.h"

/*
 * Build a new object allocating a memory area.
 */
struct object_s *
new_object(int type, char *value)
{
	struct object_s *p;

	if ((p = (struct object_s *)malloc(sizeof(struct object_s))) == NULL)
		err(1, NULL);

	if ((p->value = (char *)malloc(strlen(value) + 1)) == NULL)
		err(1, NULL);
	strcpy(p->value, value);

	p->type = type;

	p->next = NULL;
	p->assoc = NULL;

	return p;
}

/*
 * Return the last object of an object list.
 * This function assumes that p != NULL.
 */
struct object_s *
last_object(struct object_s *p)
{
	while (p->next != NULL)
		p = p->next;
	return p;
}

/*
 * Count the number of objects that are in an object list.
 */
int
count_objects(struct object_s *p)
{
	int count = 0;
	for (; p != NULL; p = p->next)
		count++;
	return count;
}

/*
 * Link an object to an object list.
 */
struct object_s *
link_object(struct object_s *dest, struct object_s *next)
{
	dest->next = next;
	return dest;
}	

/*
 * Associate an object to an object list.
 */
struct object_s *
assoc_object(struct object_s *dest, struct object_s *assoc)
{
	dest->assoc = assoc;
	return dest;
}	

/*
 * Deallocate the memory area used by an object.
 */
void
free_object(struct object_s *p)
{
	if (p->value != NULL)
		free(p->value);
	free(p);
}

/*
 * Deallocate an object list.
 */
void
free_object_list(struct object_s *p)
{
	struct object_s *next;
	while (p != NULL) {
		next = p->next;
		if (p->assoc != NULL)
			free_object_list(p->assoc);
		free_object(p);
		p = next;
	}
}
